import { pgTable, text, serial, integer, boolean, timestamp, jsonb, decimal } from "drizzle-orm/pg-core";
import { createInsertSchema } from "drizzle-zod";
import { z } from "zod";

export const users = pgTable("users", {
  id: serial("id").primaryKey(),
  username: text("username").notNull().unique(),
  email: text("email").notNull().unique(),
  password: text("password").notNull(),
  credits: integer("credits").default(100),
  subscriptionPlan: text("subscription_plan").default("basic"),
  riskScore: integer("risk_score").default(0),
  complianceLevel: text("compliance_level").default("basic"),
  createdAt: timestamp("created_at").defaultNow(),
});

export const audits = pgTable("audits", {
  id: serial("id").primaryKey(),
  userId: integer("user_id").references(() => users.id),
  type: text("type").notNull(), // internal, external, compliance, pentest, vulnerability
  title: text("title").notNull(),
  target: text("target").notNull(),
  scope: text("scope").notNull(),
  status: text("status").default("pending"), // pending, running, completed, failed
  progress: integer("progress").default(0),
  results: jsonb("results"),
  securityScore: integer("security_score"),
  vulnerabilities: integer("vulnerabilities"),
  criticalIssues: integer("critical_issues"),
  aiSummary: text("ai_summary"),
  createdAt: timestamp("created_at").defaultNow(),
  completedAt: timestamp("completed_at"),
});

export const scanResults = pgTable("scan_results", {
  id: serial("id").primaryKey(),
  userId: integer("user_id").references(() => users.id),
  type: text("type").notNull(), // email, document, network
  filename: text("filename"),
  content: text("content"),
  threatScore: integer("threat_score"),
  status: text("status").notNull(), // clean, suspicious, malicious
  findings: jsonb("findings"),
  createdAt: timestamp("created_at").defaultNow(),
});

// Assurance Services Tables
export const assuranceControls = pgTable("assurance_controls", {
  id: serial("id").primaryKey(),
  userId: integer("user_id").references(() => users.id),
  controlName: text("control_name").notNull(),
  framework: text("framework").notNull(), // ISO, SOC, HIPAA
  status: text("status").default("pending"), // validated, failed, pending
  riskLevel: text("risk_level").notNull(),
  evidence: jsonb("evidence"),
  aiRecommendations: text("ai_recommendations"),
  lastValidated: timestamp("last_validated"),
  createdAt: timestamp("created_at").defaultNow(),
});

// Cyber Insurance Tables
export const insuranceAssessments = pgTable("insurance_assessments", {
  id: serial("id").primaryKey(),
  userId: integer("user_id").references(() => users.id),
  companySize: text("company_size").notNull(),
  industry: text("industry").notNull(),
  riskFactors: jsonb("risk_factors").notNull(),
  calculatedRisk: integer("calculated_risk").notNull(),
  premiumEstimate: decimal("premium_estimate", { precision: 10, scale: 2 }),
  recommendedCoverage: jsonb("recommended_coverage"),
  aiAnalysis: text("ai_analysis"),
  createdAt: timestamp("created_at").defaultNow(),
});

// Incident Response Tables
export const incidents = pgTable("incidents", {
  id: serial("id").primaryKey(),
  userId: integer("user_id").references(() => users.id),
  title: text("title").notNull(),
  incidentType: text("incident_type").notNull(),
  severity: text("severity").notNull(),
  status: text("status").default("open"),
  description: text("description").notNull(),
  timeline: jsonb("timeline").default('[]'),
  playbook: text("playbook"),
  rootCauseAnalysis: text("root_cause_analysis"),
  aiInsights: text("ai_insights"),
  createdAt: timestamp("created_at").defaultNow(),
  resolvedAt: timestamp("resolved_at"),
});

// Security Training Tables
export const trainingModules = pgTable("training_modules", {
  id: serial("id").primaryKey(),
  title: text("title").notNull(),
  category: text("category").notNull(),
  difficulty: text("difficulty").notNull(),
  content: text("content").notNull(),
  duration: integer("duration").notNull(), // in minutes
  quizQuestions: jsonb("quiz_questions"),
  createdAt: timestamp("created_at").defaultNow(),
});

export const userTrainingProgress = pgTable("user_training_progress", {
  id: serial("id").primaryKey(),
  userId: integer("user_id").references(() => users.id),
  moduleId: integer("module_id").references(() => trainingModules.id),
  status: text("status").default("not_started"),
  progress: integer("progress").default(0),
  score: integer("score"),
  aiFeedback: text("ai_feedback"),
  completedAt: timestamp("completed_at"),
  createdAt: timestamp("created_at").defaultNow(),
});

// AI Chat Sessions
export const aiChatSessions = pgTable("ai_chat_sessions", {
  id: serial("id").primaryKey(),
  userId: integer("user_id").references(() => users.id),
  sessionType: text("session_type").notNull(), // security_coach, audit_summary, etc.
  messages: jsonb("messages").default('[]'),
  createdAt: timestamp("created_at").defaultNow(),
  updatedAt: timestamp("updated_at").defaultNow(),
});

export const services = pgTable("services", {
  id: serial("id").primaryKey(),
  userId: integer("user_id").references(() => users.id),
  type: text("type").notNull(), // assurance, insurance, incident-response, training
  status: text("status").default("active"),
  config: jsonb("config"),
  createdAt: timestamp("created_at").defaultNow(),
});

export const insertUserSchema = createInsertSchema(users).pick({
  username: true,
  email: true,
  password: true,
});

export const insertAuditSchema = createInsertSchema(audits).pick({
  type: true,
  title: true,
  target: true,
  scope: true,
});

export const insertScanResultSchema = createInsertSchema(scanResults).pick({
  type: true,
  filename: true,
  content: true,
});

export const insertServiceSchema = createInsertSchema(services).pick({
  type: true,
  config: true,
});

export const insertAssuranceControlSchema = createInsertSchema(assuranceControls).pick({
  controlName: true,
  framework: true,
  riskLevel: true,
});

export const insertInsuranceAssessmentSchema = createInsertSchema(insuranceAssessments).pick({
  companySize: true,
  industry: true,
  riskFactors: true,
});

export const insertIncidentSchema = createInsertSchema(incidents).pick({
  title: true,
  incidentType: true,
  severity: true,
  description: true,
  playbook: true,
});

export const insertTrainingProgressSchema = createInsertSchema(userTrainingProgress).pick({
  moduleId: true,
});

// Types
export type InsertUser = z.infer<typeof insertUserSchema>;
export type User = typeof users.$inferSelect;
export type InsertAudit = z.infer<typeof insertAuditSchema>;
export type Audit = typeof audits.$inferSelect;
export type InsertScanResult = z.infer<typeof insertScanResultSchema>;
export type ScanResult = typeof scanResults.$inferSelect;
export type InsertService = z.infer<typeof insertServiceSchema>;
export type Service = typeof services.$inferSelect;
export type InsertAssuranceControl = z.infer<typeof insertAssuranceControlSchema>;
export type AssuranceControl = typeof assuranceControls.$inferSelect;
export type InsertInsuranceAssessment = z.infer<typeof insertInsuranceAssessmentSchema>;
export type InsuranceAssessment = typeof insuranceAssessments.$inferSelect;
export type InsertIncident = z.infer<typeof insertIncidentSchema>;
export type Incident = typeof incidents.$inferSelect;
export type TrainingModule = typeof trainingModules.$inferSelect;
export type UserTrainingProgress = typeof userTrainingProgress.$inferSelect;
export type InsertTrainingProgress = z.infer<typeof insertTrainingProgressSchema>;
export type AiChatSession = typeof aiChatSessions.$inferSelect;
